from flask import Flask, render_template, Response, render_template_string
import cv2

app = Flask(__name__)

camera = cv2.VideoCapture(0)  # use 0 for web camera
camera.set(cv2.CAP_PROP_FRAME_WIDTH, 1280)
camera.set(cv2.CAP_PROP_FRAME_HEIGHT, 720)
camera.set(cv2.CAP_PROP_FPS, 30)


def gen_frames():  # generate frame by frame from camera
    while True:
        # Capture frame-by-frame
        success, frame = camera.read()  # read the camera frame
        if not success:
            break
        else:
            _, buffer = cv2.imencode('.jpg', frame)
            frame = buffer.tobytes()
            yield (b'--frame\r\n'
                   b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')  # concat frame one by one and show result


@app.route('/video_feed')
def video_feed():
    # Video streaming route. Put this in the src attribute of an img tag
    return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')


@app.route('/')
def index():
    """Video streaming home page."""
    return render_template_string(
        """
        <!doctype html>
<html lang="en">
<head>
    <!-- Required meta tags -->
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

    <!-- Bootstrap CSS -->
    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.1.3/css/bootstrap.min.css"
        integrity="sha384-MCw98/SFnGE8fJT3GXwEOngsV7Zt27NXFoaoApmYm81iuXoPkFOJwJ8ERdknLPMO" crossorigin="anonymous">

    <title>AIJ Live Stream</title>
</head>
<body>
<div class="container">
    <div class="row">
        <div class="col-lg-8  offset-lg-2">
            <h3 class="mt-5">Live Streaming...</h3>
            <img src="{{ url_for('video_feed') }}" width="100%">
        </div>
    </div>
</div>
</body>
</html>
"""
    )


def main():
    app.run(
        host='localhost',
        port=8080,
        debug=True
    )


if __name__ == '__main__':
    main()



import spacy

def get_entities(text):

    # load the pre-trained NER model
    nlp = spacy.load("en_core_web_sm")

    # apply named entity recognition
    doc = nlp(text)

    # extract the entities and their labels
    entities = [(ent.text, ent.label_) for ent in doc.ents]
    
    return entities

def main():
    
    # sample text
    text = "Bill Gates is the founder of Microsoft Corporation, which is based in Redmond, Washington."

    entities = get_entities(text)

    # print the results
    print("Original text:", text)
    print("Named entities:", entities)
    
    
if __name__ == "__main__":
    main()


import os, sys, datetime, time
import urllib.request

import docker

from psutil import process_iter
from signal import SIGTERM # or SIGKILL

from dotenv import load_dotenv
load_dotenv()  # take environment variables from .env.

user_profile = os.environ['USERPROFILE']
SEP = os.path.sep
ENV = os.path.join(user_profile, '.aij', '.env')

def main():
    """
    The main function to run the server and publish the news articles to the RabbitMQ queue
    """
    print('Server is being initialized...')
    
    # create an .env file to store all the environment variables if not exists
    if not os.path.exists(ENV):
        with open(ENV, 'w', encoding='utf-8') as file:
            
            # read the environment variables from the .env file
            NEWSAPI_ORG = os.environ['NEWSAPI_ORG'] if 'NEWSAPI_ORG' in os.environ else str(input('Enter your News API key: '))
            WEATHERAPI_COM = os.environ['WEATHERAPI_COM'] if 'WEATHERAPI_COM' in os.environ else str(input('Enter your Weather API key: '))
            WEATHERAPI_COM_REGION = os.environ['WEATHERAPI_COM_REGION'] if 'WEATHERAPI_COM_REGION' in os.environ else str(input('Enter your Weather API region: '))
            OPENAI_API_KEY = os.environ['OPENAI_API_KEY'] if 'OPENAI_API_KEY' in os.environ else str(input('Enter your OpenAI API key: '))
            
            # write the environment variables to the .env file
            file.write('NEWSAPI_ORG=' + NEWSAPI_ORG + '\n')
            file.write('WEATHERAPI_COM=' + WEATHERAPI_COM + '\n')
            file.write('WEATHERAPI_COM_REGION=' + WEATHERAPI_COM_REGION + '\n')
            file.write('OPENAI_API_KEY=' + OPENAI_API_KEY + '\n')
            
    
    # check first if docker is installed
    # if it has docker then run the following code
    if os.system('docker --version') == 0:
        
        print('Docker is installed!')
        
        client = docker.from_env()

        # print all containers
        containers = client.containers.list()

        for container in containers:
            print(
                container.name,
                container.id,
                container.status
            )

        # TODO: add this feature by accepting a command line argument such as --force
        # search any application running on port 5672 and close the process by its ID using sys package
        
        # if there is at least one argument then run the following code        
        # if sys.argv and len(sys.argv) > 0 and sys.argv[1] == '--force':
        
        #     if len(containers) > 0:
        #         # search if any container is running on port 5672 and close the process
        #         for container in containers:
        #             if container.name == 'aij-messaging-server':
        #                 container.kill()
                        
        #             if 5672 in container.ports:
        #                 container.kill()
                        
        #     for proc in process_iter():
        #         for conns in proc.connections(kind='inet'):
        #             if conns.laddr.port == 5672:
        #                 proc.send_signal(SIGTERM) # or SIGKILL

        client.containers.run(
            image="rabbitmq:3-management", 
            name='aij-messaging-server', 
            ports={
                '5672': 5672,
                '15672': 15672
            },
            remove = True,
            detach=True, 
        )
        
        print('Server is now running using Docker...')
        
    # if docker is not installed then run the following code
    else:
        print('Docker is not installed!')
        os.system('rabbitmq-server -detached')
        print('Server is now running using local RabbitMQ server...')

    print('Server is now running...')


if __name__ == '__main__':
    main()


from datetime import datetime
from glob import glob
import os
import sys
from typing import Optional
import threading
from deepface.basemodels.SFace import cv
import requests
from deepface import DeepFace
import cv2
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import cv2
import docker

user_profile = os.environ['USERPROFILE']
SEP = os.path.sep
LOGO_URL = "https://raw.githubusercontent.com/codesapienbe/aij-webcam/master/logo.png"
SCREEN_WIDTH = 1280
SCREEN_HEIGHT = 720
SCREEN_FPS = 30
SCREEN_START_X = 0
SCREEN_START_Y = 0
SCREEN_MAXIMIZE = True
APP_TITLE = 'AIJ'

start = datetime.now().strftime("%Y%m%d%H%M%S")
start_dt = datetime.now()
config_home = user_profile + SEP + '.aij'
config_abs = config_home + SEP + 'config.json'
image_home = user_profile + SEP + '.aij' + SEP + 'image'
logo_abs = image_home + SEP + 'logo.png'
csv_home = user_profile + SEP + '.aij' + SEP + 'news'
csv_abs = csv_home + SEP + 'news.csv'
audio_home = user_profile + SEP + '.aij' + SEP + 'audio'
screenshot_home = user_profile + SEP + '.aij' + SEP + 'screenshot'
audio_abs = audio_home + SEP + start + '.mp3'
video_home = user_profile + SEP + '.aij' + SEP + 'video'
face_home = user_profile + SEP + '.aij' + SEP + 'face'
news_df = Optional[pd.DataFrame]
weather_df = Optional[pd.DataFrame]

backends = [
    "opencv",
    "ssd",
    "dlib",
    "mtcnn",
    "retinaface",
    "mediapipe"
]

default_backend = backends[0]

models = [
    "VGG-Face",
    "Facenet",
    "Facenet512",
    "OpenFace",
    "DeepFace",
    "DeepID",
    "ArcFace",
    "Dlib",
    "SFace",
]

default_model = models[0]


def pre_init():
    """
    This function is called before the server is initialized
    """

    client = docker.from_env()
    # print all containers
    containers = client.containers.list()

    for container in containers:
        print(
            container.name,
            container.id,
            container.status
        )

    # if there is no docker container running named aij-messaging-server then run the server script 'aijinit'

    if not any(container.name == 'aij-messaging-server' for container in containers):
        os.system('aijinit')


def download_logo():
    """
    Download the logo.png file to userprofile/.aij/images/logo.png
    """

    logo_remote_response = requests.get(
        LOGO_URL, allow_redirects=True, stream=True, timeout=10)

    if logo_remote_response.status_code == 200:
        # save the logo.png file if the response is ok and the file does not exist
        if not os.path.exists(logo_abs):
            with open(logo_abs, 'wb') as logo_file:
                logo_file.write(logo_remote_response.content)


def draw_text(image, x, y, text, color):
    """
    Draw text on the image
    """

    # draw a black rectangle on the image

    cv2.rectangle(
        image,
        (x, y - 25),
        # until the end of screen width
        (x + image.shape[1], y),
        (0, 0, 0),
        -1
    )

    # draw the text on the image
    cv2.putText(
        image,
        text,
        (x, y),
        cv2.FONT_HERSHEY_SIMPLEX,
        font_size / 12,
        color,
        2,
        cv2.LINE_AA
    )


def draw_box(image, x, y, color):
    """
    Draw a box on the image
    """
    cv2.rectangle(
        image,
        (x, y),
        (x + box_size, y + box_size),
        color,
        2
    )


def draw_button(image, x, y, text, color):
    """
    Draw a button on the image and write text on it
    """

    cv2.rectangle(
        image,
        (x, y),
        (x + box_size, y + box_size),
        color,
        2
    )

    cv2.putText(
        image,
        text,
        (x + 5, y + 30),
        cv2.FONT_HERSHEY_SIMPLEX,
        font_size,
        color,
        2,
        cv2.LINE_AA
    )


def draw_zoomed_text(image, x, y, text, color):
    """
    Draw zoomed text on the image
    """
    cv2.putText(
        image,
        text,
        (x, y),
        cv2.FONT_HERSHEY_SIMPLEX,
        font_size,
        color,
        2,
        cv2.LINE_AA
    )


def draw_logo(logo_path, img):
    if os.path.exists(logo_path):
        logo = cv2.imread(logo_path)
        # Resize the logo to match the height of the main image and keep the aspect ratio
        # resize the logo to 100px height and keep the aspect ratio
        logo_resized = cv2.resize(
            logo, (48, 48), interpolation=cv2.INTER_AREA)
        # get the height and width of the logo
        logo_height, logo_width, _ = logo_resized.shape
        # calculate the x and y coordinates of the logo
        x = 5
        y = 5
        # draw the logo on the main image
        img[y:y + logo_height, x:x + logo_width] = logo_resized
        return img


def draw_emoji(current_emotion, img):
    emoji_home = user_profile + SEP + '.aij' + SEP + 'emojis'

    # find all emojis
    matching_emoji = [emoji for emoji in os.listdir(emoji_home) if emoji.endswith(
        ".png") and emoji.startswith(current_emotion)]
    emj = cv2.imread(emoji_home + SEP + matching_emoji[0])

    # resize the emj to 64px height and keep the aspect ratio
    emj_resized = cv2.resize(emj, (64, 64), interpolation=cv2.INTER_AREA)

    # get the height and width of the emj
    emj_height, emj_width, _ = emj_resized.shape
    # get the height and width of the main image
    image_height, image_width, _ = img.shape
    # calculate the x and y coordinates of the emj
    x = image_width - emj_width - 5
    y = image_height - emj_height - 5
    # draw the emj on the main image
    img[y:y + emj_height, x:x + emj_width] = emj_resized
    return img


# do not wait for the directories to be created
thread_download_logo = threading.Thread(target=download_logo)
thread_download_logo.start()

default_backend = "opencv"
default_model = "VGG-Face"

cam = cv2.VideoCapture(0)
cam.set(cv2.CAP_PROP_FRAME_WIDTH, SCREEN_WIDTH)
cam.set(cv2.CAP_PROP_FRAME_HEIGHT, SCREEN_HEIGHT)
cam.set(cv2.CAP_PROP_FPS, SCREEN_FPS)

# Define min window size to be recognized as a face
MIN_FACE_FRAME_WIDTH = 50
MIN_FACE_FRAME_HEIGHT = 50
FRAME_COUNTER = 0
current_emotion = "neutral"

while cam.isOpened():

    ret, img = cam.read()
    img = cv2.flip(img, 1)  # Flip vertically
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    # increase the frame counter
    FRAME_COUNTER += 1

    try:
        # after each 5 seconds detect faces
        if FRAME_COUNTER >= SCREEN_FPS:
            # reset the frame counter
            FRAME_COUNTER = 0
            # predict the emotion
            emotion = DeepFace.analyze(img_path=img, actions=[
                'emotion'], detector_backend=default_backend, align=True, silent=True)

            # get the emotion data from the dictionary
            emotion_data = emotion[0]['emotion']
            # convert to dataframe
            emotion_df = pd.DataFrame(emotion_data, index=[0])
            # get the dominant emotion
            emotion_label = emotion_df.idxmax(axis=1)[0]
            # add the emotion to the emotion history
            current_emotion = emotion_label

    except Exception as e:
        # draw error message if no face detected
        pass

    img = draw_emoji(current_emotion, img)
    img = draw_logo(logo_abs, img)

    cv2.imshow('Emotion Recognition', img)

    if cv2.waitKey(1) & 0xFF == ord('q'):
        cam.release()
        break


if cam.isOpened():
    cam.release()

cv2.destroyAllWindows()



import nltk

nltk.download()

# Tokenization - Breaking text into words or sentences
text = "Natural language processing is a challenging field, but it can also be very rewarding."
sentences = nltk.sent_tokenize(text)
words = nltk.word_tokenize(text)

print(sentences)
print(words)
pos_tags = nltk.pos_tag(words)

print(pos_tags)

# Named Entity Recognition - Identifying named entities (such as names, places, and organizations) in text
ner_tags = nltk.ne_chunk(pos_tags)

print(ner_tags)


import cv2
import mediapipe as mp

mp_drawing = mp.solutions.drawing_utils
mp_objectron = mp.solutions.objectron

def main():

    # For webcam input:
    cap = cv2.VideoCapture(0)

    with mp_objectron.Objectron(static_image_mode=False,
                                max_num_objects=5,
                                min_detection_confidence=0.5,
                                min_tracking_confidence=0.75,
                                model_name='Cup') as objectron:
        while cap.isOpened():
            success, image = cap.read()
            if not success:
                print("Ignoring empty camera frame.")
                # If loading a video, use 'break' instead of 'continue'.
                continue

            # To improve performance, optionally mark the image as not writeable to
            # pass by reference.
            image.flags.writeable = False
            image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
            results = objectron.process(image)

            # Draw the box landmarks on the image.
            image.flags.writeable = True
            image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
            if results.detected_objects:
                for detected_object in results.detected_objects:
                    mp_drawing.draw_landmarks(
                        image, detected_object.landmarks_2d, mp_objectron.BOX_CONNECTIONS)
                    mp_drawing.draw_axis(image, detected_object.rotation,
                                        detected_object.translation)
            # Flip the image horizontally for a selfie-view display.
            cv2.imshow('Objectron', cv2.flip(image, 1))

            
            # if escape or q is pressed, break the loop
            if (cv2.waitKey(5) & 0xFF == 27) or (cv2.waitKey(5) & 0xFF == ord('q')):
                break
        

    cap.release()
    cv2.destroyAllWindows()
    
    
if __name__ == '__main__':
    main()



# Define a knowledge base as a dictionary
knowledge_base = {
    "artificial intelligence": ["AI", "machine learning", "neural networks"],
    "climate change": ["global warming", "greenhouse gases", "carbon emissions"],
    "COVID-19": ["coronavirus", "pandemic", "vaccine"],
    # and so on...
}

# Define a function that takes a topic and returns related terms from the knowledge base
def get_related_terms(topic):
    if topic in knowledge_base:
        return knowledge_base[topic]
    else:
        return []
    
def main():

    # Test the function with a few different topics
    print(get_related_terms("artificial intelligence"))

    print(get_related_terms("COVID-19"))

    print(get_related_terms("space exploration"))
    
    
if __name__ == "__main__":
    main()


import time
import pika
import os
import openai
import docker

from dotenv import load_dotenv

load_dotenv()  # take environment variables from .env.

# TODO: Implement this feature later
def pre_init():
    """
    This function is called before the server is initialized
    """
    client = docker.from_env()

    # print all containers
    containers = client.containers.list()

    for container in containers:
        print(
            container.name,
            container.id,
            container.status
        )

    # if there is no docker container running named aij-messaging-server then run the server script 'aijinit'
    if not any(container.name == 'aij-messaging-server' for container in containers):
        os.system('aijinit')

class AIAssistant:
    """
    AI Assistant class that uses OpenAI's GPT-3 API to generate responses.
    """

    def __init__(self, api_key):
        """
        Initializes the AI Assistant.
        :param api_key: The OpenAI API key.
        """
        self.api_key = api_key
        self.start_sequence = "\nAIJ:"
        self.restart_sequence = "\nHuman: "
        self.model = "text-davinci-003"
        self.temperature = 1
        self.max_tokens = 4000
        self.top_p = 1
        self.frequency_penalty = 0.62
        self.presence_penalty = 0.6
        self.stop = [" Human:", " AI:"]

        openai.api_key = self.api_key

    def generate_response(self, prompt):
        """
        Generates a response using OpenAI's GPT-3 API.
        :param prompt: The prompt to use for the AI to generate a response.
        :return: The generated response.
        """
        response = openai.Completion.create(
            model=self.model,
            prompt=prompt,
            temperature=self.temperature,
            max_tokens=self.max_tokens,
            top_p=self.top_p,
            frequency_penalty=self.frequency_penalty,
            presence_penalty=self.presence_penalty,
            stop=self.stop
        )
        return response.choices[0].titles


class NewsConsumer:
    """
    News Consumer class that consumes news from a RabbitMQ queue.
    """

    def __init__(self, host, ai_assistant):
        """
        Initializes the News Consumer.
        """
        self.ai_assistant = ai_assistant
        self.connection = pika.BlockingConnection(
            pika.ConnectionParameters(host))
        self.channel = self.connection.channel()
        self.channel.queue_declare(queue='news_stream')

    def consume(self):
        """
        Consumes news from the RabbitMQ queue.
        """
        self.channel.basic_consume(
            queue='news_stream', on_message_callback=self.callback)
        self.channel.start_consuming()

    def callback(self, ch, method, properties, body):
        """
        Callback function that is called when a message is received from the RabbitMQ queue.
        :param ch: The channel.
        :param method: The method.
        :param properties: The properties.
        :param body: The body of the message.
        """
        prompt = body.decode('utf-8').strip() + \
            self.ai_assistant.restart_sequence
        response = self.ai_assistant.generate_response(prompt)
        print(response)
        ch.basic_ack(delivery_tag=method.delivery_tag)

    def close(self):
        """
        Closes the connection.
        """
        if self.connection.is_open:
            self.connection.close()


def main():
    
    api_key = os.environ.get("OPENAI_API_KEY")
    ai_assistant = AIAssistant(api_key)
    consumer = NewsConsumer('localhost', ai_assistant)

    try:
        consumer.consume()
    except KeyboardInterrupt:
        consumer.close()


if __name__ == '__main__':
    main()



import cv2
import mediapipe as mp
import numpy as np

mp_drawing = mp.solutions.drawing_utils
mp_selfie_segmentation = mp.solutions.selfie_segmentation

def main():

    # For static images:
    IMAGE_FILES = []
    BG_COLOR = (192, 192, 192)  # gray
    MASK_COLOR = (255, 255, 255)  # white
    with mp_selfie_segmentation.SelfieSegmentation(
            model_selection=0) as selfie_segmentation:
        for idx, file in enumerate(IMAGE_FILES):
            image = cv2.imread(file)
            image_height, image_width, _ = image.shape
            # Convert the BGR image to RGB before processing.
            results = selfie_segmentation.process(
                cv2.cvtColor(image, cv2.COLOR_BGR2RGB))

            # Draw selfie segmentation on the background image.
            # To improve segmentation around boundaries, consider applying a joint
            # bilateral filter to "results.segmentation_mask" with "image".
            condition = np.stack((results.segmentation_mask,) * 3, axis=-1) > 0.1
            # Generate solid color images for showing the output selfie segmentation mask.
            fg_image = np.zeros(image.shape, dtype=np.uint8)
            fg_image[:] = MASK_COLOR
            bg_image = np.zeros(image.shape, dtype=np.uint8)
            bg_image[:] = BG_COLOR
            output_image = np.where(condition, fg_image, bg_image)
            cv2.imwrite('/tmp/selfie_segmentation_output' +
                        str(idx) + '.png', output_image)

    # For webcam input:
    BG_COLOR = (192, 192, 192)  # gray
    cap = cv2.VideoCapture(0)
    with mp_selfie_segmentation.SelfieSegmentation(
            model_selection=1) as selfie_segmentation:
        bg_image = None
        while cap.isOpened():
            success, image = cap.read()
            if not success:
                print("Ignoring empty camera frame.")
                # If loading a video, use 'break' instead of 'continue'.
                continue

            # Flip the image horizontally for a later selfie-view display, and convert
            # the BGR image to RGB.
            image = cv2.cvtColor(cv2.flip(image, 1), cv2.COLOR_BGR2RGB)
            # To improve performance, optionally mark the image as not writeable to
            # pass by reference.
            image.flags.writeable = False
            results = selfie_segmentation.process(image)

            image.flags.writeable = True
            image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)

            # Draw selfie segmentation on the background image.
            # To improve segmentation around boundaries, consider applying a joint
            # bilateral filter to "results.segmentation_mask" with "image".
            condition = np.stack(
                (results.segmentation_mask,) * 3, axis=-1) > 0.1
            # The background can be customized.
            #   a) Load an image (with the same width and height of the input image) to
            #      be the background, e.g., bg_image = cv2.imread('/path/to/image/file')
            #   b) Blur the input image by applying image filtering, e.g.,
            #      bg_image = cv2.GaussianBlur(image,(55,55),0)
            if bg_image is None:
                bg_image = np.zeros(image.shape, dtype=np.uint8)
                bg_image[:] = BG_COLOR
            output_image = np.where(condition, image, bg_image)

            cv2.imshow('MediaPipe Selfie Segmentation', output_image)
            if cv2.waitKey(5) & 0xFF == 27:
                break

    cap.release()
    cv2.destroyAllWindows()
    
if __name__ == '__main__':
    main()
